"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const path_1 = __importDefault(require("path"));
const postcss_1 = __importDefault(require("postcss"));
const postcss_import_sync2_1 = __importDefault(require("postcss-import-sync2"));
const postcss_icss_selectors_1 = __importDefault(require("postcss-icss-selectors"));
const utils_1 = require("./utils");
function init({ typescript: ts }) {
    const create = (info) => {
        // prettier-ignore
        const processor = (0, postcss_1.default)()
            .use((0, postcss_import_sync2_1.default)())
            .use((0, postcss_icss_selectors_1.default)());
        const { createLanguageServiceSourceFile } = ts;
        ts.createLanguageServiceSourceFile = (fileName, scriptSnapshot, ...rest) => {
            if ((0, utils_1.isStyle)(fileName)) {
                scriptSnapshot = (0, utils_1.getDtsSnapshot)(ts, scriptSnapshot, fileName, processor);
            }
            const sourceFile = createLanguageServiceSourceFile(fileName, scriptSnapshot, ...rest);
            if ((0, utils_1.isStyle)(fileName)) {
                sourceFile.isDeclarationFile = true;
            }
            return sourceFile;
        };
        const { updateLanguageServiceSourceFile } = ts;
        ts.updateLanguageServiceSourceFile = (sourceFile, scriptSnapshot, ...rest) => {
            const { fileName } = sourceFile;
            if ((0, utils_1.isStyle)(fileName)) {
                scriptSnapshot = (0, utils_1.getDtsSnapshot)(ts, scriptSnapshot, fileName, processor);
            }
            sourceFile = updateLanguageServiceSourceFile(sourceFile, scriptSnapshot, ...rest);
            if ((0, utils_1.isStyle)(fileName)) {
                sourceFile.isDeclarationFile = true;
            }
            return sourceFile;
        };
        if (info.languageServiceHost.resolveModuleNames) {
            const { resolveModuleNames } = info.languageServiceHost;
            info.languageServiceHost.resolveModuleNames = (moduleNames, containingFile, ...rest) => {
                const resolvedModules = resolveModuleNames.call(info.languageServiceHost, moduleNames, containingFile, ...rest);
                return moduleNames.map((moduleName, index) => {
                    if ((0, utils_1.isStyle)(moduleName) && (0, utils_1.isRelative)(moduleName)) {
                        const fileName = path_1.default.resolve(path_1.default.dirname(containingFile), moduleName);
                        return {
                            extension: ts.Extension.Dts,
                            isExternalLibraryImport: false,
                            resolvedFileName: fileName,
                        };
                    }
                    else {
                        return resolvedModules[index];
                    }
                });
            };
        }
        return info.languageService;
    };
    const getExternalFiles = (project) => {
        return project.getFileNames().filter((name) => (0, utils_1.isStyle)(name));
    };
    return { create, getExternalFiles };
}
module.exports = init;
