"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDtsSnapshot = exports.isRelative = exports.isStyle = exports.isStyl = exports.isLess = exports.isSass = exports.isCss = void 0;
const icss_utils_1 = require("icss-utils");
const tags_1 = require("./tags");
const sass_1 = __importDefault(require("sass"));
const stylus_1 = __importDefault(require("stylus"));
const less_1 = __importDefault(require("less"));
const CANDY_SIGN = '// Candy';
const isCss = (fileName) => fileName.endsWith('.css');
exports.isCss = isCss;
const isSass = (fileName) => fileName.endsWith('.scss') || fileName.endsWith('.sass');
exports.isSass = isSass;
const isLess = (fileName) => fileName.endsWith('.less');
exports.isLess = isLess;
const isStyl = (fileName) => fileName.endsWith('.styl');
exports.isStyl = isStyl;
const isStyle = (fileName) => (0, exports.isCss)(fileName) || (0, exports.isSass)(fileName) || (0, exports.isLess)(fileName) || (0, exports.isStyl)(fileName);
exports.isStyle = isStyle;
const isRelative = (fileName) => /^\.\.?($|[\\/])/.test(fileName);
exports.isRelative = isRelative;
const getDtsSnapshot = (ts, scriptSnapshot, fileName, processor) => {
    const source = scriptSnapshot.getText(0, scriptSnapshot.getLength());
    if (source.startsWith(CANDY_SIGN)) {
        return scriptSnapshot;
    }
    const css = transformSourceToCss(fileName, source);
    const classnames = getClassnames(fileName, css, processor);
    const dts = [];
    dts.push(CANDY_SIGN);
    dts.push(`import type { ComponentProps } from 'react'`);
    dts.push(`type Props = {`);
    dts.push('[k: `__${string}`]: string | number');
    if (classnames.length) {
        for (const classname of classnames) {
            dts.push(`${classname}?: boolean`);
        }
    }
    else {
        dts.push(`[k: string]?: any`);
    }
    dts.push(`}`);
    dts.push(`type Component<T extends keyof JSX.IntrinsicElements> = (props: ComponentProps<T> & Props) => JSX.IntrinsicElements<T>`);
    for (let i = 0; i < tags_1.tags.length; i++) {
        const tag = tags_1.tags[i];
        const component = tags_1.components[i];
        dts.push(`export const ${component}: Component<'${tag}'>`);
    }
    dts.push(`declare const styles: {`);
    for (const classname of classnames) {
        dts.push(`${classname}: string`);
    }
    dts.push(`}`);
    dts.push(`export default styles`);
    return ts.ScriptSnapshot.fromString(dts.join(`\n`));
};
exports.getDtsSnapshot = getDtsSnapshot;
const transformSourceToCss = (fileName, source) => {
    if ((0, exports.isSass)(fileName)) {
        const { css } = sass_1.default.compile(fileName);
        return css;
    }
    if ((0, exports.isLess)(fileName)) {
        let css;
        less_1.default.render(source, {
            syncImport: true,
            filename: fileName,
        }, (error, output) => {
            if (error)
                throw error;
            css = output ? output.css.toString() : '';
        });
        return css;
    }
    if ((0, exports.isStyl)(fileName)) {
        const css = (0, stylus_1.default)(source, {
            filename: fileName,
        }).render();
        return css;
    }
    if ((0, exports.isCss)(fileName)) {
        return source;
    }
    throw new Error('Unkown file type');
};
const getClassnames = (fileName, css, processor) => {
    const compiled = processor.process(css, {
        from: fileName,
    });
    if (compiled.root) {
        const extracted = (0, icss_utils_1.extractICSS)(compiled.root);
        return Object.keys(extracted.icssExports);
    }
    return [];
};
